//#include <MIDI.h>





// Vibe Bar Main Program
//
// Sees the movement of the Vibe Bar
//  Between full left and dead spot
//  Between full right and dead spot
//  Between full forward and dead spot
//  Between full back and dead spot
// and maps to MIDI output.
//
//  Claude Woodward 17/06/15
//

#include <Vibe_Bar_Configuration.h>
#include <EEPROM.h>

// MIDI Libraries
#include <MIDI.h>
#include <midi_Defs.h>
#include <midi_Message.h>
#include <midi_Namespace.h>
#include <midi_Settings.h>

//const int AUX_BC = 0;
//const int AUX_PEDAL = 1;
//const int NUM_AUX = 2;
//const int statusLEDpin = 1;
const int statusLEDpin = 11;
// Aux input
//int auxMode = AUX_BC;
//int val = 0;
//int lastVal = 0;
int lastBend = 0;
//int potPin = A2;
int MIDIChannel = 4;
unsigned long debounceTime = 0;
unsigned long auxDebounceTime = 1000;
unsigned long modeChangeTime = 0;
int modeFlashNum = 0;
unsigned long flashLength = 200;
int ledState = 1;

byte ctrlChange = 180;
//byte noteON = 144;//note on command
//byte note;//storage for currently playing note



//MIDI_CREATE_DEFAULT_INSTANCE();

// Time to hold button on startup for calibration mode
int calModeTime = 2000;

// These Analog pins are defined in ~/Documents/Arduino/libraries/Vibe_Bar_Configuration/Vibe_Bar_Configuration.cpp
// These lines tell the Arduino where the Hall Effect sensors are connected to
//const int SENSOR_LEFT_RIGHT = A0;
//const int SENSOR_FWD_BACK = A1;
extern const int SENSOR_LEFT_RIGHT;
extern const int SENSOR_FWD_BACK;
// These Analog pins are defined in ~/Documents/Arduino/libraries/Vibe_Bar_Configuration/Vibe_Bar_Configuration.cpp
// This defines where the Arduino built-in LED is
extern const int LED_PIN;

// This digital pins is defined in ~/Documents/Arduino/libraries/Vibe_Bar_Configuration/Vibe_Bar_Configuration.cpp
extern const int PUSH_BUTTON;
extern const bool BUTTON_NATURAL_STATE;

// These are text names for non-volatile memory addresses
// These are defined in ~/Documents/Arduino/libraries/Vibe_Bar_Configuration/Vibe_Bar_Configuration.cpp
extern const int VIBE_LR_MIN1;
extern const int VIBE_LR_MIN2;
extern const int VIBE_LR_MAX1;
extern const int VIBE_LR_MAX2;
extern const int VIBE_LR_DEAD_MIN1;
extern const int VIBE_LR_DEAD_MIN2;
extern const int VIBE_LR_DEAD_MAX1;
extern const int VIBE_LR_DEAD_MAX2;
extern const int VIBE_FB_MIN1;
extern const int VIBE_FB_MIN2;
extern const int VIBE_FB_MAX1;
extern const int VIBE_FB_MAX2;
extern const int VIBE_FB_DEAD_MIN1;
extern const int VIBE_FB_DEAD_MIN2;
extern const int VIBE_FB_DEAD_MAX1;
extern const int VIBE_FB_DEAD_MAX2;


// CC definitions

#include "FastLED.h"
#define DATA_PIN 5

// Define the array of leds
#define NUM_LEDS 16
CRGB leds[NUM_LEDS];

// Baud rate (31250 is MIDI baud rate)
#define data_rate 31250
//Mux control pins

int s0 = 1;
int s1 = 2;
int s2 = 3;
int s3 = 4;



int controlPin[] = {s0, s1, s2, s3};

//Mux in "SIG" pin
int SIG_pin1 = A2;


// The number of milliseconds between successive sends of the CC data.

#define cclowestChan 108
#define chanPerMUX 16


struct CCPotStruct {
  int CCnum;//channel number
  int CCval;//current value
  int CCprev;//previous value
  int CCtype;
  //set to 0 if channel is not used.
  //set to 1 if pot
  //set to 2 if switch
  //set to 4 if joystick

};
CCPotStruct CCPot1[chanPerMUX];

long lastScan = 0;



// This is called a 'class' it contains functions and data that alre all carried around together
// This particular class contains the graph mapping the hall effect sensor reading to a MIDI output
class CustomCalibration {
  public:
    void setup( int minVal, int maxVal, int deadMin, int deadMax, int outMin, int outMax ) {
      _minVal = minVal;
      _maxVal = maxVal;
      _deadMin = deadMin;
      _deadMax = deadMax;
      _outMin = outMin;
      _outMax = outMax;
      _outMid = ( outMin + outMax ) / 2;
      if ( minVal > deadMin || deadMin > deadMax || deadMax > maxVal || outMin > outMax )
      {
        pinMode(LED_PIN, OUTPUT);
        Serial.begin(19200);
        Serial.println("EEPROM Calibration Value Check Failed: Possible cause = bad calibration");
        while (true) {
          digitalWrite(LED_PIN, HIGH);
          delay(500);
          digitalWrite(LED_PIN, LOW);
          delay(500);
          Serial.print(" minVal = ");
          Serial.print(_minVal);
          Serial.print(" maxVal = ");
          Serial.print(_maxVal);
          Serial.print(" deadMin = ");
          Serial.print(_deadMin);
          Serial.print(" deadMax = ");
          Serial.print(_deadMax);
          Serial.print(" outMin = ");
          Serial.print(_outMin);
          Serial.print(" outMax = ");
          Serial.print(_outMax);
          Serial.println();
        }
      }
      MIDI.begin();
    }

    int Map(int value) {
      if ( value < _minVal ) {
        return _outMin;
      } else if ( value < _deadMin ) {
        return map( value, _minVal, _deadMin, _outMin, _outMid );
      } else if ( value < _deadMax ) {
        return _outMid;
      } else if ( value < _maxVal ) {
        return map( value, _deadMax, _maxVal, _outMid, _outMax );
      } else {
        return _outMax;
      }
    }
  private:
    int _minVal, _maxVal, _deadMin, _deadMax, _outMin, _outMax, _outMid;
};

CustomCalibration leftRightCalibration, fwdBackCalibration;

void setup() 
{ 
  

  checkForCalibration();

  // Initialise the Serial port for MIDI 31250 (MIDI is sent over Serial)
  Serial.begin(31250);

  // Create custom mapping objects (they map the Hall Effect sensor values to the MIDI range - and incorporate a 'dead spot')
  leftRightCalibration = CustomCalibration();
  fwdBackCalibration = CustomCalibration();

  // MIDI Range constants
  int midiMin = 0;
  int midiMax = 127;

  // Load calibration points from the EEPROM (these are stored during the calibration routine)
  leftRightCalibration.setup( EEPROM.read(VIBE_LR_MIN1) + EEPROM.read(VIBE_LR_MIN2) * 256, EEPROM.read(VIBE_LR_MAX1) + EEPROM.read(VIBE_LR_MAX2) * 256,
                              EEPROM.read(VIBE_LR_DEAD_MIN1) + EEPROM.read(VIBE_LR_DEAD_MIN2) * 256, EEPROM.read(VIBE_LR_DEAD_MAX1) + EEPROM.read(VIBE_LR_DEAD_MAX2) * 256, midiMin, ( ( midiMax * 2 ) / 3 ) + 1 );
  fwdBackCalibration.setup( EEPROM.read(VIBE_FB_MIN1) + EEPROM.read(VIBE_FB_MIN2) * 256, EEPROM.read(VIBE_FB_MAX1) + EEPROM.read(VIBE_FB_MAX2) * 256,
                            EEPROM.read(VIBE_FB_DEAD_MIN1) + EEPROM.read(VIBE_FB_DEAD_MIN2) * 256, EEPROM.read(VIBE_FB_DEAD_MAX1) + EEPROM.read(VIBE_FB_DEAD_MAX2) * 256, midiMin, ( midiMax * 1 ) / 3 );

  // Load MIDI Channel from EEPROM
  MIDIChannel = EEPROM.read(VIBE_MIDI_CHANNEL);

  // Check for valid MIDI Channel
  if (MIDIChannel >= 1 && MIDIChannel <= 16) {
    // good
  }
  else
  {
    // Bad MIDI Channel so set to 1
    MIDIChannel = 4;

    // store back to EEPROM
    EEPROM.write(VIBE_MIDI_CHANNEL, 4);
  }

  //CC 1 OCT Code
  FastLED.addLeds<WS2812B, DATA_PIN>(leds, NUM_LEDS);
  FastLED.setBrightness(90);
  pinMode(s0, OUTPUT);
  pinMode(s1, OUTPUT);
  pinMode(s2, OUTPUT);
  pinMode(s3, OUTPUT);
  //pinMode(LEDpin, OUTPUT);

 
  digitalWrite(s0, LOW);
  digitalWrite(s1, LOW);
  digitalWrite(s2, LOW);
  digitalWrite(s3, LOW);


  //initialise dead channel flags to 0
  for (int x = 0; x < chanPerMUX; x++)
  {
    CCPot1[x].CCtype = 1;

  }

  //THESE ARE YOUR SWITCH CHANNELS!!!
  //MUX 1 = CH 0 to CH 15
  CCPot1[8].CCtype = 4; //Breath Control
  CCPot1[9].CCtype = 2; //Floor Switch 1
  CCPot1[10].CCtype = 2;//Floor Switch 2
  CCPot1[11].CCtype = 3;//Pedal
  CCPot1[12].CCtype = 0;
  CCPot1[13].CCtype = 0;
  CCPot1[14].CCtype = 0;
  CCPot1[15].CCtype = 0;



}

void checkForCalibration() {

  // Initialise the push-button and tell the Arduino it is an input
  pinMode(PUSH_BUTTON, INPUT_PULLUP);

  // Check if button is pressed
  int start = millis();
  if (digitalRead(PUSH_BUTTON) != BUTTON_NATURAL_STATE) {
    // Wait until button is released
    while (digitalRead(PUSH_BUTTON) != BUTTON_NATURAL_STATE) {}

    // Button released
    int duration = millis() - start;

    // Perform calibration if button pressed for sufficently long
    if (duration > calModeTime) {
      calibrate();

      // Calibration done, now spin forever until rebooted
      while (true) {}
    }

  }
}

// Value used during calibration
int sensorValue;

void calibrate() {

  // This is the initialisation routine so that text sent from the Arduino
  // can be read on the computer
  Serial.begin(19200);
  blinkLED(statusLEDpin);

  // Send the following text to the computer
  Serial.println("Vibe Bar Cal Routine");

  Serial.println("move bar full left and then press the button");
  WaitUntilButtonIsPressed( PUSH_BUTTON );
  sensorValue = analogRead(SENSOR_LEFT_RIGHT);
  Serial.print("Sensor value is ");
  Serial.println(sensorValue);
  EEPROM.write(VIBE_LR_MIN1, sensorValue % 256);
  EEPROM.write(VIBE_LR_MIN2, sensorValue / 256);
  WaitUntilButtonIsReleased( PUSH_BUTTON );
  blinkLED(statusLEDpin);

  Serial.println("move bar full right and then press the button");
  WaitUntilButtonIsPressed( PUSH_BUTTON );
  sensorValue = analogRead(SENSOR_LEFT_RIGHT);
  Serial.print("Sensor value is ");
  Serial.println(sensorValue);
  EEPROM.write(VIBE_LR_MAX1, sensorValue % 256);
  EEPROM.write(VIBE_LR_MAX2, sensorValue / 256);
  WaitUntilButtonIsReleased( PUSH_BUTTON );
  blinkLED(statusLEDpin);

  Serial.println("move just left of dead zone and then press the button");
  WaitUntilButtonIsPressed( PUSH_BUTTON );
  sensorValue = analogRead(SENSOR_LEFT_RIGHT);
  Serial.print("Sensor value is ");
  Serial.println(sensorValue);
  EEPROM.write(VIBE_LR_DEAD_MIN1, sensorValue % 256);
  EEPROM.write(VIBE_LR_DEAD_MIN2, sensorValue / 256);
  WaitUntilButtonIsReleased( PUSH_BUTTON );
  blinkLED(statusLEDpin);

  Serial.println("move just right of dead zone and then press the button");
  WaitUntilButtonIsPressed( PUSH_BUTTON );
  sensorValue = analogRead(SENSOR_LEFT_RIGHT);
  Serial.print("Sensor value is ");
  Serial.println(sensorValue);
  EEPROM.write(VIBE_LR_DEAD_MAX1, sensorValue % 256);
  EEPROM.write(VIBE_LR_DEAD_MAX2, sensorValue / 256);
  WaitUntilButtonIsReleased( PUSH_BUTTON );
  blinkLED(statusLEDpin);

  Serial.println("move bar full forward and then press the button");
  WaitUntilButtonIsPressed( PUSH_BUTTON );
  sensorValue = analogRead(SENSOR_FWD_BACK);
  Serial.print("Sensor value is ");
  Serial.println(sensorValue);
  EEPROM.write(VIBE_FB_MAX1, sensorValue % 256);
  EEPROM.write(VIBE_FB_MAX2, sensorValue / 256);
  WaitUntilButtonIsReleased( PUSH_BUTTON );
  blinkLED(statusLEDpin);

  Serial.println("move bar full back and then press the button");
  WaitUntilButtonIsPressed( PUSH_BUTTON );
  sensorValue = analogRead(SENSOR_FWD_BACK);
  Serial.print("Sensor value is ");
  Serial.println(sensorValue);
  EEPROM.write(VIBE_FB_MIN1, sensorValue % 256);
  EEPROM.write(VIBE_FB_MIN2, sensorValue / 256);
  WaitUntilButtonIsReleased( PUSH_BUTTON );
  blinkLED(statusLEDpin);

  Serial.println("move just forward of dead zone and then press the button");
  WaitUntilButtonIsPressed( PUSH_BUTTON );
  sensorValue = analogRead(SENSOR_FWD_BACK);
  Serial.print("Sensor value is ");
  Serial.println(sensorValue);
  EEPROM.write(VIBE_FB_DEAD_MAX1, sensorValue % 256);
  EEPROM.write(VIBE_FB_DEAD_MAX2, sensorValue / 256);
  WaitUntilButtonIsReleased( PUSH_BUTTON );
  blinkLED(statusLEDpin);

  Serial.println("move just back of dead zone and then press the button");
  WaitUntilButtonIsPressed( PUSH_BUTTON );
  sensorValue = analogRead(SENSOR_FWD_BACK);
  Serial.print("Sensor value is ");
  Serial.println(sensorValue);
  EEPROM.write(VIBE_FB_DEAD_MIN1, sensorValue % 256);
  EEPROM.write(VIBE_FB_DEAD_MIN2, sensorValue / 256);
  WaitUntilButtonIsReleased( PUSH_BUTTON );
  blinkLED(statusLEDpin);
  blinkLED(statusLEDpin);
  blinkLED(statusLEDpin);

  Serial.println("The calibration is complete.");
}

// Functions to handle button presses for calibration
const int DEBOUNCE_MS = 250;
void WaitUntilButtonIsPressed( int button ) {
  delay(DEBOUNCE_MS);
  while ( digitalRead(button) == BUTTON_NATURAL_STATE )
  {
    // Do nothing until the button is pressed
  }
}
void WaitUntilButtonIsReleased( int button ) {
  delay(DEBOUNCE_MS);
  while ( digitalRead(button) != BUTTON_NATURAL_STATE )
  {
    // Do nothing until the button is released
  }
}

void blinkLED (int LEDpin)
{
  digitalWrite(LEDpin, HIGH);
  delay(250);
  digitalWrite(LEDpin, LOW);
  delay(250);
  digitalWrite(LEDpin, HIGH);
  delay(250);
  digitalWrite(LEDpin, LOW);
}



void loop()
{
  

  if (ledState == 1)
  {
    digitalWrite(statusLEDpin, HIGH);
  }
  else
  {
    digitalWrite(statusLEDpin, LOW);
  }


  // Read Sensors

  int leftRightRawValue = analogRead( SENSOR_LEFT_RIGHT );
  int fwdBackRawValue = analogRead( SENSOR_FWD_BACK );

  // Map to MIDI range
  int leftRightValue = leftRightCalibration.Map( leftRightRawValue );
  int fwdBackValue = fwdBackCalibration.Map( fwdBackRawValue );

  // Combine Left-Right and Forward-Back into a single bend measurement
  int bendVal = leftRightValue + fwdBackValue;

  // Send as MIDI command
  if (abs( bendVal - lastBend) >= 1)
  {
    int tempBend = bendVal;
    usbMIDI.sendPitchBend(tempBend << 7, 4);
    //MIDI.sendPitchBend((bendVal << 7) - 8065, MIDIChannel);

    lastBend = bendVal;
  }
  /*
  usbMIDI.sendPitchBend(bendVal << 7, MIDIChannel);
  MIDI.sendPitchBend((bendVal << 7) - 8065, MIDIChannel);

  // Button to change channel
  if (digitalRead(PUSH_BUTTON) != BUTTON_NATURAL_STATE)
  {
    // Ensure we dont toggle too much!
    if (millis() - debounceTime > auxDebounceTime)
    {
      // Next MIDI Channel
      MIDIChannel = MIDIChannel + 1;

      if (MIDIChannel == 17)
      {
        MIDIChannel = 1;
      }

      // Store channel in EEPROM
      EEPROM.write(VIBE_MIDI_CHANNEL, MIDIChannel);

      //set up some bits and pieces to flash the LED depending on the state.
      modeChangeTime = millis();//This sets the start time for the blink
      modeFlashNum = MIDIChannel;

      // Reset button debounce
      debounceTime = millis();    //Each time through the loop, check if it is time to change the LED state and if there are any blinks left to do.
    }
  }
*/
  

  //Each time through the loop, check if it is time to change the LED state and if there are any blinks left to do.
  if (millis() - modeChangeTime > flashLength && modeFlashNum > 0)
  {
    ledState = !ledState;//toggle the flag that sets the LED on or off
    if (ledState == 1)//If the led is back on, decrement the blink counter by 1.
    {
      modeFlashNum = modeFlashNum - 1;
    }
    modeChangeTime = millis();//finally, reset the timer ready for the next blink
  }
 */

  // CC 1 oct Code

  for (int i = 0; i < chanPerMUX; i++)
  {
    CCPot1[i].CCnum = i + cclowestChan;
    CCPot1[i].CCprev = CCPot1[i].CCval;
    if (CCPot1[i].CCtype == 0)//cancel data on dead channels
    {
      CCPot1[i].CCval = 0;
      
    }
    else if (CCPot1[i].CCtype == 1)//sliding things - soft/membrane pots
    {
      int val = readMux(i, SIG_pin1, s0);
      if(val > 0) {
        CCPot1[i].CCval = map(val, 50, 980, 0, 127);
        CCPot1[i].CCval = constrain(CCPot1[i].CCval, 0, 127);   
        //smoothing
        CCPot1[i].CCval = (CCPot1[i].CCprev * 2 + CCPot1[i].CCval) / 3;
      } else {
        CCPot1[i].CCval = CCPot1[i].CCprev;
      }
    }
    
    else if (CCPot1[i].CCtype == 2)//switch type mapping
    {
      CCPot1[i].CCval = readMux(i, SIG_pin1, s0);
      if (CCPot1[i].CCval > 700)
      {
        CCPot1[i].CCval = 126;
      }
      else
      {
        CCPot1[i].CCval = 0;
      }
    }
      else if (CCPot1[i].CCtype == 3)//pot type mapping
    {
      CCPot1[i].CCval = map(readMux(i, SIG_pin1, s0), 540, 700, 0, 127);
      CCPot1[i].CCval = constrain(CCPot1[i].CCval, 0, 127);  
      //smoothing
      CCPot1[i].CCval = (CCPot1[i].CCprev * 2 + CCPot1[i].CCval) / 3;
    }
     else if (CCPot1[i].CCtype == 4)//pot type mapping
    {
      CCPot1[i].CCval = map(readMux(i, SIG_pin1, s0), 100, 800, 0, 127);
      CCPot1[i].CCval = constrain(CCPot1[i].CCval, 0, 127);  
      //smoothing
      CCPot1[i].CCval = (CCPot1[i].CCprev * 2 + CCPot1[i].CCval) / 3;
    }
    
  
    if (i < 8) {
      leds[i*2] = CHSV(map (CCPot1[i].CCval, 0, 127, 150, 0), 255, 255);
      leds[i*2+1] = CHSV(map (CCPot1[i].CCval, 0, 127, 150, 0), 255, 255);
    }
    
    if (abs(CCPot1[i].CCval - CCPot1[i].CCprev) >= 1)
    {
      usbMIDI.sendControlChange (CCPot1[i].CCnum , CCPot1[i].CCval,4);
      // Serial.println(CCPot1[i].CCval);
     //leds[i] = CHSV(map (CCPot1[i].CCval, 0, 127, 150, 0), 255, 255);
        //lastScan = timeNOW;
        
    }
  }
  
  FastLED.show();



  delay(20);

}



int readMux(int channel, int Apin, int Cpin)
//channel = the MUX channel currently being read
//Apin = the Analogue pin the MUX output is attached to
//Cpin = the first control pin.  The control pins MUST be sequential (e,g, pins 0,1,2,3 NOT 0,3,5,8)
{
  int muxChannel[16][4] =
  {
    {0, 0, 0, 0}, //channel 0
    {1, 0, 0, 0}, //channel 1
    {0, 1, 0, 0}, //channel 2
    {1, 1, 0, 0}, //channel 3
    {0, 0, 1, 0}, //channel 4
    {1, 0, 1, 0}, //channel 5
    {0, 1, 1, 0}, //channel 6
    {1, 1, 1, 0}, //channel 7
    {0, 0, 0, 1}, //channel 8
    {1, 0, 0, 1}, //channel 9
    {0, 1, 0, 1}, //channel 10
    {1, 1, 0, 1}, //channel 11
    {0, 0, 1, 1}, //channel 12
    {1, 0, 1, 1}, //channel 13
    {0, 1, 1, 1}, //channel 14
    {1, 1, 1, 1}, //channel 15

  };

  for (int i = 0; i < 4; i ++)
  {
    digitalWrite(controlPin[i], muxChannel[channel][i]);
  }

  //read the value at the signal pin
  int val = analogRead(Apin);

  //return the value
  return val;
}
